@@ -48,6 +48,7 @@
     Contact: maxime.perrotin@esa.int
 """
 
+__all__ = ['Symbol', 'VerticalSymbol', 'HorizontalSymbol', 'Comment']
 import os
 import sys
 import logging
@@ -434,7 +435,8 @@ class Symbol(QObject, QGraphicsPathItem, object):
     # By default symbol size may expand when inner text exceeds border
     auto_expand = True
     # By default connections between symbols are lines, not arrows
-    arrow_head = False
+    arrow_head = None
+    arrow_tail = None
     # Default mouse cursor
     default_cursor = Qt.SizeAllCursor
     # Decide if a symbol can be copy-pasted several times
@@ -857,7 +859,6 @@ class Symbol(QObject, QGraphicsPathItem, object):
         '''
         # Save current position to be able to revert move
         self.coord = self.pos()
-        #event_pos = event.pos()
         rect = self.boundingRect()
         self.height = rect.height()
         if self.grabber.resize_mode != '':
@@ -873,7 +874,7 @@ class Symbol(QObject, QGraphicsPathItem, object):
     def mouse_move(self, event):
         ''' Handle resizing of items - moving is handled in subclass '''
         self.updateConnectionPoints()
-        # If any, update movable end points of connections 
+        # If any, update movable end points of connections
         for point in self.movable_points:
             point.edge.end_connection.update_position()
         if self.mode == 'Resize':
