@@ -173,6 +173,7 @@ class PrimPath(Primary):
     is_raw = False
 
 class PrimVariable(PrimPath): pass # XXX should not be raw for codegen
+class PrimFPAR(PrimVariable): pass
 class PrimEnumeratedValue(Primary): pass
 class PrimInteger(Primary): pass
 class PrimReal(Primary): pass
@@ -268,7 +269,7 @@ class Answer(object):
     ''' AST Entry for a decision answer '''
     def __init__(self):
         ''' One ANSWER of a DECISION '''
-        self.inputString = 'case'
+        self.inputString = ''
         self.line = None
         self.charPositionInLine = None
         self.pos_x = 0
@@ -303,7 +304,7 @@ class Task(object):
     ''' AST Entry for TASKS '''
     def __init__(self):
         ''' Initialize TASK attributes (set of ASSIGN statements) '''
-        self.inputString = 'x := 1'
+        self.inputString = ''
         self.line = None
         self.charPositionInLine = None
         self.pos_x = 0
@@ -342,7 +343,7 @@ class TaskForLoop(Task):
 
 class Output(object):
     ''' AST Entry for OUTPUT statements '''
-    def __init__(self, defName='RI'):
+    def __init__(self, defName=''):
         ''' Set of OUTPUT statements '''
         self.inputString = defName
         self.pos_x = 0
@@ -371,7 +372,7 @@ class ProcedureCall(Output):
 
 class Terminator(object):
     ''' Terminator elements (join, nextstate, stop) '''
-    def __init__(self, defName='Wait'):
+    def __init__(self, defName=''):
         ''' Initialize terminator attributes '''
         self.inputString = defName
         self.pos_x = None
@@ -380,7 +381,7 @@ class Terminator(object):
         self.height = 35
         self.line = None
         self.charPositionInLine = None
-        # one of 'next_state' 'join' 'stop'
+        # one of 'next_state' 'join' 'stop', 'return'
         self.kind = None
         # optional comment symbol
         self.comment = None
@@ -388,6 +389,21 @@ class Terminator(object):
         self.hyperlink = None
         # optional Label instance (to be placed just before the terminator)
         self.label = None
+        # Return expression
+        self.return_expr = None
+        # via clause, used for entering nested state with an entry point
+        # 'via' is the string for the renderer (e.g. "via foo")
+        self.via = None
+        # 'entrypoint' is the string of the entry point (e.g. "foo")
+        self.entrypoint = None
+        # some transitions can be chained, when entering/leaving nested states
+        self.next_id = -1
+        # Pointer to the next transition, when using return/connect
+        self.next_trans = None
+        # List of State that can lead to this terminator
+        # There can be several if terminator follows a floating label
+        # Note, this field is updated by the Helper.flatten function
+        self.possible_states = []
 
     def __repr__(self):
         ''' Debug output for terminators '''
@@ -402,7 +418,7 @@ class Label(object):
     def __init__(self):
         ''' Initialize the label attributes '''
         # inputString holds the label name
-        self.inputString = 'Here'
+        self.inputString = ''
         self.pos_x = 0
         self.pos_y = 0
         self.width = 70
@@ -456,7 +472,12 @@ class Transition(object):
         self.terminator = None
         # All Terminators of this transition
         self.terminators = []
-
+        # List of State that can lead to this transition
+        # There can be several if state has multiple names (e.g. STATE a, b)
+        # Note, this field is updated by the Helper.flatten function
+        # and is needed to properly know when to call a nested state exit
+        # procedure when the model is flattened.
+        self.possible_states = []
 
     def __repr__(self):
         ''' Debug output: display all actions '''
@@ -471,7 +492,7 @@ class Input(object):
     def __init__(self):
         ''' Initialize the Input attributes '''
         # inputString is the user text, it can contain several inputs
-        self.inputString = 'PI'
+        self.inputString = ''
         self.pos_x = 0
         self.pos_y = 0
         self.width = 70
@@ -502,6 +523,21 @@ class Input(object):
                 l=self.line, c=self.charPositionInLine)
 
 
+class Connect(Input):
+    ''' AST Entry for the CONNECT part (transition below a nested state) '''
+    def __init__(self):
+        ''' Only one difference with INPUT: the connect_list attribute '''
+        super(Connect, self).__init__()
+        # List of strings
+        self.connect_list = []
+        self.width = 1
+
+    def __repr__(self):
+        ''' Debug output for a CONNECT symbol '''
+        return 'CONNECT {exp} ({l},{c})'.format(exp=self.inputString,
+                l=self.line, c=self.charPositionInLine)
+
+
 class Start(object):
     ''' AST Entry for the START symbol '''
     def __init__(self):
@@ -522,7 +558,7 @@ class Start(object):
 
     def __repr__(self):
         ''' Debug output for a START symbol '''
-        return 'START'
+        return 'START {}'.format(self.inputString)
 
 
 class Procedure_start(Start):
@@ -530,6 +566,11 @@ class Procedure_start(Start):
     pass
 
 
+class CompositeState_start(Start):
+    ''' Composite state start symbol - inherits from Start, can have a name '''
+    pass
+
+
 class Comment(object):
     ''' AST Entry for COMMENT symbols '''
     def __init__(self):
@@ -567,10 +608,14 @@ class State(object):
         self.height = 35
         # list of type Input
         self.inputs = []
+        # list of type Connect (connection below a nested state)
+        self.connects = []
         # optional comment symbol
         self.comment = None
         # optional hyperlink
         self.hyperlink = None
+        # optional composite state content (type CompositeState)
+        self.composite = None
 
     def __repr__(self):
         ''' Debug output for a STATE symbol '''
@@ -584,7 +629,7 @@ class TextArea(object):
     def __init__(self):
         ''' Text area (raw content for rendering only) '''
         self.inputString = '-- Declare your variables\n\n' \
-                           '-- Syntax: DCL <variable name> <type name>;'
+                           '-- Syntax: DCL <variable name> <type name>;\n\n'
         # DCL variables in the text area {name: (sort, default_value), ...}
         self.variables = {}
         self.line = None
@@ -613,13 +658,14 @@ class Automaton(object):
         self.start = None
         self.floating_labels = []
         self.states = []
+        self.named_start = []
 
 
 class Procedure(object):
     ''' Internal procedure definition '''
     def __init__(self):
         ''' Procedure AST default value '''
-        self.inputString = 'Proc'
+        self.inputString = ''
         self.line = None
         self.charPositionInLine = None
         # Set default coordinates and width/height
@@ -630,12 +676,16 @@ class Procedure(object):
         self.hyperlink = None
         # Local variables dictionnary (see Process)
         self.variables = {}
-        # Inherited variables from all levels above
+        self.timers = []
+        # Inherited variables and timers from all levels above
         self.global_variables = {}
+        self.global_timers = []
         # Formal parameters - list of dict:
         # [{'name': str, 'direction': 'in'/'out', 'type': str}]
         self.fpar = []
-        # start, states, terminators, text areas, floating_labels (see Process)
+        # return type (ASN.1)
+        self.return_type = None
+        # start, terminators, text areas, floating_labels (see Process)
         #self.start = None
         #self.states = []
         self.terminators = []
@@ -643,9 +693,9 @@ class Procedure(object):
         # Keep a list of labels and floating labels - useful for backends
         self.labels = []
         #self.floating_labels = []
-        # Inherited procedures and operators XXX should be lists no?
-        self.procedures = {}
-        self.operators = {}
+        # Inherited procedures and operators
+        self.procedures = []
+        self.operators = []
         # inner procedures and operators (see Process for format)
         #self.inner_procedures = []
         self.inner_operators = []
@@ -673,8 +723,17 @@ class Process(object):
         self.referenced = False
         # variables: dictionnary: {variable1Name: (asn1SccType, default value)}
         self.variables = {}
-        # global variables can be used to inherit variables
+        # global variables and timers can be used to inherit from a level above
         self.global_variables = {}
+        self.global_timers = []
+        # Set default coordinates and width/height
+        self.pos_x = self.pos_y = 150
+        self.width = 150
+        self.height = 75
+        # Optional hyperlink
+        self.hyperlink = None
+        # Optional comment
+        self.comment = None
 
         # dataview: complete AST of the ASN.1 types
         self.asn1Modules = None
@@ -697,7 +756,6 @@ class Process(object):
         # list of Procedure (external procedures)
         self.procedures = []
 
-
         # The Mapping structure should be used for code generation backends
         # dictionnary: {'stateName': [class Input1, Input2,...], ...}
         # then Input contains the inputs list and corresponding transition
@@ -706,6 +764,9 @@ class Process(object):
         # list of type Transition - use 'mapping' to map index to inputs/states
         self.transitions = []
 
+        # list of type CompositeState
+        self.composite_states = []
+
         # Set of symbols contained in the process (type Automaton)
         # (Includes inner procedures)
         self.content = Automaton(parent=self)
@@ -714,6 +775,31 @@ class Process(object):
         self.timers = []
 
 
+class CompositeState(Process):
+    '''
+        Composite states: the difference with Process is that they can have:
+        - several START elements, that correspond to state entry points,
+        - state exit points (with RETURN terminators)
+        - entry and exit procedures
+    '''
+    def __init__(self):
+        super(CompositeState, self).__init__()
+        self.statename = ''
+        self.state_entrypoints = []
+        self.state_exitpoints = []
+        # Special entry and exit procedures (named "entry" and "exit")
+        self.entry_procedure = None
+        self.exit_procedure = None
+        # Body can contain text areas, procedures, composite states,
+        # one nameless START, named START (one per entrypoint), states,
+        # amd floating labels
+
+    def __repr__(self):
+        ''' Debug output for composite state '''
+        return 'COMPOSITE STATE {exp} ({l},{c})'.format(exp=self.statename,
+                l=self.line, c=self.charPositionInLine)
+
+
 class Block(object):
     ''' AST for a BLOCK entity '''
     def __init__(self):
